package edgebundles.visualisation.tree;

import java.awt.Graphics;

import edgebundles.tree.*;
import java.awt.Point;
import java.awt.Color;

/**
 * <p>Title: InvertedRadialLayoutVisualiser</p>
 *
 * <p>Description: This layout is similar to the radial layout, but it reverses
 * the placement of the nodes, so that the root is in the most outer level
 * and the leaves are in the most inner level. It offers the advantage that an
 * inner circle is vacated offering good place to visualise adjacency relations,
 * and a desirable separation between hierachical and adjacency relations.
 *
 * Note 1: the logical (output) locations of the nodes are preserved as in the
 * radial layout, only the visualisation is inverted.
 * This enables thee links to follow the virtual (non-inverted) heirarchy
 * in the vacated inner place.
 * Note 2: this layout shoulbe be used only for trees where the adjacency
 * relations takes place only between leaves. Otherwise, a non-desirable
 * visalisation may occur, making a link originate from a node-free location...
 * </p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * <p>Company: </p>
 *
 * @author Bilal Alsallakh
 * @version 1.0
 */
public class InvertedRadialLayoutVisualiser extends TreeVisualiser {

    public String toString() {
        return "Inverted Radial Layout";
    }

    private int outerRStep, innerRStep, innerR;
    private double alphaStep;
    Graphics g;
    int centerX, centerY;

    public void visualise(VisualTree tree, Graphics g, int width, int height) {
        currAngle = maxLevel = 0;
        this.centerX = width / 2;
        this.centerY = height / 2;
        this.g = g;
        layoutTree(tree.getRoot(), 0);
        int dim = Math.min(height, width);
        innerR = 4 * dim / 10;
        innerRStep = innerR / maxLevel;
        outerRStep = (dim / 2 - innerR) / maxLevel;
        alphaStep = 360d / currAngle;
        g.setColor(Color.BLACK);
        for (int i = 0; i < tree.getRoot().getChildrenCount(); i++) {
            drawNode(tree.getRoot().getChild(i), 1);
        }
    }

    int currAngle, maxLevel;

    void drawNode(TreeNode node, int level) {
        Point p = node.getCenter();
        int amin = (int) (p.x * alphaStep);
        int amax = (int) (p.y * alphaStep);
        double aminRadian = amin * Math.PI / 180,
                            amaxRadian = amax * Math.PI / 180;
        int x;
        int y;
        double cosAMid = Math.cos((aminRadian + amaxRadian) / 2);
        double sinAMid = Math.sin((aminRadian + amaxRadian) / 2);
        if (node.getChildrenCount() == 0) {
            int ro = innerR + (maxLevel - level + 1) * outerRStep
                     + (level == 1 ? -3 : 3);
            x = (int) (innerR * cosAMid) + centerX;
            y = (int) (innerR * sinAMid) + centerY;
            int x1 = (int) (ro * cosAMid) + centerX;
            int y1 = (int) (ro * sinAMid) + centerY;
            g.drawLine(x, y, x1, y1);
            g.fillArc(x - 3, y - 3, 6, 6, 0, 360);
        } else {
            x = (int) (level * innerRStep * cosAMid) + centerX;
            y = (int) (level * innerRStep * sinAMid) + centerY;
            for (int i = 0; i < node.getChildrenCount(); i++) {
                drawNode(node.getChild(i), level + 1);
            }
            double cos1 = Math.cos(aminRadian), cos2 = Math.cos(amaxRadian),
                    sin1 = Math.sin(aminRadian), sin2 = Math.sin(amaxRadian);
            int ri = innerR + (maxLevel - level) * outerRStep + 3;
            int ro = innerR + (maxLevel - level + 1) * outerRStep - 3;
            int x1 = (int) (ri * cos1) + centerX;
            int y1 = (int) (ri * sin1) + centerY;
            int x2 = (int) (ri * cos2) + centerX;
            int y2 = (int) (ri * sin2) + centerY;
            int x3 = (int) (ro * cos1) + centerX;
            int y3 = (int) (ro * sin1) + centerY;
            int x4 = (int) (ro * cos2) + centerX;
            int y4 = (int) (ro * sin2) + centerY;
            g.drawArc(centerX - ri, centerY - ri, 2 * ri, 2 * ri, -amin,
                      amin - amax);
            g.drawArc(centerX - ro, centerY - ro, 2 * ro, 2 * ro, -amin,
                      amin - amax);
            g.drawLine(x1, y1, x3, y3);
            g.drawLine(x2, y2, x4, y4);
        }
        p.setLocation(x, y);
    }


    void layoutTree(TreeNode node, int level) {
        if (level > maxLevel) {
            maxLevel = level;
        }
        if (node.getChildrenCount() == 0) {
            node.setCenter(currAngle, currAngle + 1);
            currAngle += 2;
        } else {
            TreeNode child = node.getChild(0);
            layoutTree(child, level + 1);
            int minAngle = child.getCenter().x;
            for (int i = 1; i < node.getChildrenCount(); i++) {
                child = node.getChild(i);
                layoutTree(child, level + 1);
            }
            int maxAngle = child.getCenter().y;
            node.setCenter(minAngle, maxAngle);
        }
    }

}
